@@ -3,32 +3,49 @@ from funk.error import FunkyError
 from funk.call import Call
 from funk.call import IntegerCallCount
 from funk.call import InfiniteCallCount
+from funk.sequence import Sequence
 from funk.util import function_call_str
 
 __all__ = ['with_context', 'Context', 'expects', 'allows', 'set_attr', 'expects_call', 'allows_call']
 
+class UnexpectedInvocationError(AssertionError):
+    def __init__(self, mock_name, args, kwargs, expectations):
+        args_str = map(repr, args)
+        kwargs_str = {}
+        for key, value in kwargs.iteritems():
+            kwargs_str[key] = repr(value)
+        call_str = function_call_str(mock_name, args_str, kwargs_str)
+        exception_str = ["Unexpected invocation: %s" % call_str]
+        exception_str.append("\nThe following expectations on %s did not match:\n    " % mock_name)
+        if len(expectations) > 0:
+            exception_str.append("\n    ".join(e.replace("\n", "\n    ") for e in expectations))
+        else:
+            exception_str.append("No expectations set.")
+        super(UnexpectedInvocationError, self).__init__(''.join(exception_str))
+
 class Mock(object):
     def __init__(self, base, name):
-        self._name = name
         self._mocked_calls = MockedCalls(base, name)
+        self._base = base
         
     def __getattribute__(self, name):
         my = lambda name: object.__getattribute__(self, name)
         mocked_calls = my('_mocked_calls')
-        if name in mocked_calls:
+        base = my('_base')
+        if name in mocked_calls or (base is not None and hasattr(base, name)):
             return mocked_calls.for_method(name)
         return my(name)
         
     def __call__(self, *args, **kwargs):
-        return self._mocked_calls.for_self()(*args, **kwargs)
+        return object.__getattribute__(self, "_mocked_calls").for_self()(*args, **kwargs)
         
     def _verify(self):
-        self._mocked_calls.verify()
+        object.__getattribute__(self, "_mocked_calls").verify()
 
 class MockedCalls(object):
     def __init__(self, base, mock_name):
         self._base = base
-        self._method_calls = []
+        self._method_calls = {}
         self._function_calls = []
         self._mock_name = mock_name
     
@@ -38,8 +55,12 @@ class MockedCalls(object):
                 raise AssertionError("Method '%s' is not defined on type object '%s'" % (method_name, self._base.__name__))
             if not callable(getattr(self._base, method_name)):
                 raise AssertionError("Attribute '%s' is not callable on type object '%s'" % (method_name, self._base.__name__))
-        call = Call(method_name, call_count)
-        self._method_calls.append(call)
+        call = Call("%s.%s" % (self._mock_name, method_name), call_count)
+        
+        if method_name not in self._method_calls:
+            self._method_calls[method_name] = []
+        
+        self._method_calls[method_name].append(call)
         return call
     
     def add_function_call(self, call_count):
@@ -48,24 +69,24 @@ class MockedCalls(object):
         return call
     
     def for_method(self, name):
-        method_calls = filter(lambda call: call.has_name(name), self._method_calls)
-        return MockedCallsForFunction("%s.%s" %(self._mock_name, name), method_calls)
+        return MockedCallsForFunction("%s.%s" % (self._mock_name, name), self._method_calls.get(name, []))
     
     def for_self(self):
         return MockedCallsForFunction(self._mock_name, self._function_calls)
     
     def __contains__(self, name):
-        return any([call.has_name(name) for call in self._method_calls])
+        return name in self._method_calls
         
     def verify(self):
-        for call in self._method_calls:
-            self._verify_call(call, "%s.%s" % (self._mock_name, call))
+        for method_name in self._method_calls:
+            for call in self._method_calls[method_name]:
+                self._verify_call(call)
         for call in self._function_calls:
-            self._verify_call(call, call)
+            self._verify_call(call)
                 
-    def _verify_call(self, call, name):
+    def _verify_call(self, call):
         if not call.is_satisfied():
-            raise AssertionError("Not all expectations were satisfied. Expected call: %s" % name)
+            raise AssertionError("Not all expectations were satisfied. Expected call: %s" % call)
 
 class MockedCallsForFunction(object):
     def __init__(self, name, calls):
@@ -73,12 +94,12 @@ class MockedCallsForFunction(object):
         self._calls = calls
         
     def __call__(self, *args, **kwargs):
+        desc = []
         for call in self._calls:
-            if call.accepts(args, kwargs):
+            if call.accepts(args, kwargs, desc):
                 return call(*args, **kwargs)
         
-        call_str = function_call_str(self._name, args, kwargs)
-        raise AssertionError("Unexpected invocation: %s" % call_str)
+        raise UnexpectedInvocationError(self._name, args, kwargs, desc)
 
 def with_context(test_function, mock_factory=None):
     @wraps(test_function)
@@ -113,22 +134,22 @@ class ExpectationCreator(object):
 def expects(mock, method_name=None):
     if method_name is None:
         return ExpectationCreator(lambda method_name: expects(mock, method_name))
-    return mock._mocked_calls.add_method_call(method_name, IntegerCallCount(1))
+    return object.__getattribute__(mock, "_mocked_calls").add_method_call(method_name, IntegerCallCount(1))
 
 def allows(mock, method_name=None):
     if method_name is None:
         return ExpectationCreator(lambda method_name: allows(mock, method_name))
-    return mock._mocked_calls.add_method_call(method_name, InfiniteCallCount())
+    return object.__getattribute__(mock, "_mocked_calls").add_method_call(method_name, InfiniteCallCount())
     
 def set_attr(mock, **kwargs):
     for kwarg in kwargs:
         setattr(mock, kwarg, kwargs[kwarg])
 
 def expects_call(mock):
-    return MethodArgumentsSetter(mock._mocked_calls.add_function_call(IntegerCallCount(1)))
+    return MethodArgumentsSetter(object.__getattribute__(mock, "_mocked_calls").add_function_call(IntegerCallCount(1)))
 
 def allows_call(mock):
-    return MethodArgumentsSetter(mock._mocked_calls.add_function_call(InfiniteCallCount()))
+    return MethodArgumentsSetter(object.__getattribute__(mock, "_mocked_calls").add_function_call(InfiniteCallCount()))
 
 class Context(object):
     def __init__(self, mock_factory=None):
@@ -137,11 +158,28 @@ class Context(object):
         self._mocks = []
         self._mock_factory = mock_factory
     
-    def mock(self, base=None, name='unnamed'):
-        mock = self._mock_factory(base, name)
+    def mock(self, base=None, name=None):
+        mock = self._mock_factory(base, self._generate_name(name, base))
         self._mocks.append(mock)
         return mock
         
     def verify(self):
         for mock in self._mocks:
             mock._verify()
+            
+    def sequence(self):
+        return Sequence()
+        
+    def _generate_name(self, name, base):
+        if name is not None:
+            return name
+        if base is None:
+            return "unnamed"
+        name = []
+        name.append(base.__name__[0].lower())
+        for character in base.__name__[1:]:
+            if character.isupper():
+                name.append("_%s" % character.lower())
+            else:
+                name.append(character)
+        return ''.join(name)
