@@ -1,9 +1,9 @@
 from nose.tools import assert_equals
 
-def assert_raises_str(exception, message, function):
+def assert_raises_str(exception, message, function, *args, **kwargs):
     passed = True
     try:
-        function()
+        function(*args, **kwargs)
         passed = False
     except exception, e:
         assert_equals(message, str(e))
@@ -17,8 +17,18 @@ def assert_that(value, matcher):
 
 class ValueObject(object):
     def __init__(self, attributes):
+        self._keys = attributes.keys()
         for key in attributes:
             setattr(self, key, attributes[key])
 
+    def __str__(self):
+        attributes = {}
+        for key in self._keys:
+            attributes[key] = getattr(self, key)
+        return "<value_object: %s>" % attributes
+    
+    def __repr__(self):
+        return str(self)
+
 def value_object(**kwargs):
     return ValueObject(kwargs)
