@@ -29,6 +29,13 @@ def test_assert_raises_str_fails_if_messages_do_not_match():
         raise TypeError("Oh dear.")
     assert_raises(AssertionError, lambda: assert_raises_str(TypeError, "Oh noes!", func))
 
+def test_assert_raises_str_can_take_arguments_for_function_under_test():
+    def func(name, number):
+        if name == "Sir Galahad" and number == 42:
+            raise RuntimeError("Look out!")
+            
+    assert_raises(RuntimeError, lambda: assert_raises_str(TypeError, "Oh noes!", func, "Sir Galahad", number=42))
+
 def test_assert_that_passes_if_matcher_returns_true():
     class TrueMatcher(Matcher):
         def matches(self, value, failure_out):
@@ -64,3 +71,16 @@ def test_value_object_sets_attributes_to_passed_keyword_arguments():
     obj = value_object(width=20, height=40)
     assert_equals(obj.width, 20)
     assert_equals(obj.height, 40)
+
+def test_value_object_str_shows_attributes():
+    obj = value_object(width=20)
+    assert_equals(str(obj), "<value_object: {'width': 20}>")
+
+def test_value_object_str_shows_updated_attributes():
+    obj = value_object(width=20)
+    obj.width = 30
+    assert_equals(str(obj), "<value_object: {'width': 30}>")
+
+def test_value_object_repr_is_same_as_str():
+    obj = value_object(width=20)
+    assert_equals(repr(obj), "<value_object: {'width': 20}>")
